package cluster;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import constants.Constant;


import similarity.LTC_LTC;
import similarity.VSM;
import window.FileTooShort;
import index.DirectoryNotFound;
import index.FileID;
import index.Thesaurus;

/**
 * Cluster the documents, we start with each document as a cluster we merge two nearest clusters which
 * their similarity is more than some Threshold, in each step. The similarity is based on VSM and for now it's
 * based on LTC_LTC. Because the behavior of the weighting scheme is polymorphic change is very easy.
 */
public class Clustering {
	public static void main(String[] args) {
		try {
			Thesaurus thesauri = new Thesaurus(Constant.getString("Clustering.HomeDirecory"));
			FileID[] fileIDs = thesauri.getFileIDs();
			VSM model = new LTC_LTC(thesauri);
			List clusters = new ArrayList();
			boolean goOn = true;
			
			for (int i = 0; i < fileIDs.length; i++) {
				String fileName = fileIDs[i].getFileName();

				clusters.add( new Cluster(fileName, thesauri.getDocumentTerms( fileName)));
			}

			while(goOn) {
				goOn = false;
				
				double[][] similarity = new double[clusters.size()][clusters.size()];
				double max = -1;
				int forMerge1 = 0;
				int forMerge2 = 0;
				
				for (int i = 0; i < clusters.size(); i++) {
					Cluster cluster1 = (Cluster)clusters.get(i);
					
					for (int j = 0; j < i; j++) {
						Cluster cluster2 = (Cluster)clusters.get(j);
						
						similarity[i][j] = model.similarity(cluster1.getTerms(), cluster2.getTerms());
					}
				}
				
				for (int i = 0; i < similarity.length; i++) {
					for (int j = 0; j < i; j++) {
						if(similarity[i][j] > max) {
							goOn = true;
							forMerge1 = i;
							forMerge2 = j;
							max = similarity[i][j];
						}
					}
				}
				
				if(goOn) {
					if(max >= Float.parseFloat(Constant.getString("Clustering.Threshold"))) {
//						System.out.println("I: " + forMerge1 + " " + ((Cluster)clusters.get(forMerge1)).getFileNames());
//						System.out.println("J: " + forMerge2 + " " + ((Cluster)clusters.get(forMerge2)).getFileNames());
						((Cluster)clusters.get(forMerge1)).merge((Cluster)clusters.get(forMerge2));
						clusters.remove(forMerge2);
					} else {
						goOn = false;
					}
				}
			}
			
			for (Iterator it = clusters.iterator(); it.hasNext();) {
				Cluster cluster = (Cluster) it.next();
				
				System.out.print("[");
				
				for (Iterator iter = cluster.getFileNames().iterator(); iter.hasNext();) {
					String fileName = (String) iter.next();
					
					System.out.print(fileName);
					
					if(iter.hasNext())
						System.out.print(", ");
				}
				
				System.out.println("]");
			}
			
		} catch (DirectoryNotFound e) {
			System.out.println("Directory must exist!!");
			System.out.println();
			e.printStackTrace(System.err);
			System.exit(1);
		} catch (FileTooShort e) {
			System.out.println("One of the files is too short!!");
			System.out.println();
			e.printStackTrace(System.err);
			System.exit(1);
		}
	}
}
