package similarity;

import java.util.Iterator;
import java.util.List;

import org.mortbay.util.MultiMap;

import window.FileTooShort;
import window.Fingerprint;

import index.DirectoryNotFound;
import index.FileID;
import index.IndexEntry;
import index.Thesaurus;

/**
 * The similarity computed using a heuristic; percent of matching terms
 */
public class ComputeSimilarity {
	public static void main(String[] args) {
		try {
			Thesaurus thesauri = new Thesaurus("files");
			
			FileID[] fileIDs = thesauri.getFileIDs();
			MultiMap index = thesauri.getInvertedIndex();
			
			float[][] result = new float[fileIDs.length][fileIDs.length];
			
			for (int i = 0; i < fileIDs.length; i++) {
				for (int j = 0; j < result[i].length; j++)
					result[i][j] = 0;
					
				Fingerprint[] fingerprints = fileIDs[i].getFingerprints();
				
				for (int j = 0; j < fingerprints.length; j++) {
					Integer fp = new Integer( fingerprints[j].getValue());
					
					boolean[] ownerDocuments = new boolean[fileIDs.length];
					
					for (int k = 0; k < ownerDocuments.length; k++)
						ownerDocuments[k] = false;
					
					List otherFingerprints = index.getValues(fp);
					
					//TODO optimize finding files
					for (Iterator iter = otherFingerprints.iterator(); iter.hasNext();) {
						IndexEntry other = (IndexEntry) iter.next();
						
						for (int k = 0; k < fileIDs.length; k++) {
							if( fileIDs[k].getFileName().equals(other.getFileName())) {
								ownerDocuments[k] = true;
								break;
							}
						}
					}
					
					for (int k = 0; k < ownerDocuments.length; k++) {
						if( ownerDocuments[k])
							result[i][k]++;
					}
					
				}

				for (int k = 0; k < result[i].length; k++)
					result[i][k] /= fingerprints.length;
			}
			
			for (int i = 0; i < fileIDs.length; i++) {
				System.out.println(fileIDs[i].getFileName() + " " + fileIDs[i].getFingerprints().length);
				
				for (int j = 0; j < result[i].length; j++)
					System.out.println("\t"+ "<" + fileIDs[j].getFileName() + " , " + result[i][j] + ">");
			}

			//Temp
//			System.out.println(index);
//			
//			Iterator it = index.keySet().iterator();
//			it.next();
//			it.next();
//			it.next();
//			it.next();
//			it.next();
//			Integer myi = (Integer) it.next();
//
//			System.out.println(index.getValues(myi));
//			System.out.println(InvertedIndex.getDocuments(myi, index));
		} catch (DirectoryNotFound e) {
			System.out.println("Directory must exist!!");
			e.printStackTrace(System.err);
			System.exit(1);
		} catch (FileTooShort e) {
			System.out.println("One of the files is too short!!");
			e.printStackTrace(System.err);
			System.exit(1);
		}
	}
}
